const ffmpeg = require('fluent-ffmpeg');
const fs = require('fs');
const path = require('path');
const { exec } = require('child_process');

module.exports = function(RED) {
    function ExtractKeyframesNode(config) {
        RED.nodes.createNode(this, config);
        const node = this;

        // 查找系统中FFmpeg的路径
        exec('which ffmpeg', (err, stdout, stderr) => {
            if (err || !stdout) {
                node.on('input', function(msg) {
                    msg.payload = 'FFmpeg not found. Please install FFmpeg and ensure it is in your PATH.';
                    node.send(msg);
                });
                return;
            }

            const ffmpegPath = stdout.trim();
            ffmpeg.setFfmpegPath(ffmpegPath);

            node.on('input', function(msg) {
                const videoPath = msg.path || config.videoPath;
                const outputDir = config.outputDir || './keyframes';
                const frameRate = config.frameRate || 1;

                if (!videoPath) {
                    msg.payload = 'No video path specified. Please provide a video path in msg.path or configure it in the node settings.';
                    node.send(msg);
                    return;
                }

                if (!fs.existsSync(outputDir)) {
                    fs.mkdirSync(outputDir, { recursive: true });
                }

                const outputPattern = path.join(outputDir, 'frame-%04d.png');
                const outputFiles = [];

                ffmpeg(videoPath)
                    .output(outputPattern)
                    .outputOptions([
                        `-vf fps=${frameRate}`
                    ])
                    .on('end', function() {
                        fs.readdir(outputDir, (err, files) => {
                            if (err) {
                                msg.payload = 'Error reading output directory: ' + err.message;
                                node.send(msg);
                                return;
                            }
                            files.forEach(file => {
                                if (file.startsWith('frame-') && file.endsWith('.png')) {
                                    outputFiles.push(path.join(outputDir, file));
                                }
                            });
                            msg.payload = outputFiles;
                            node.send(msg);
                        });
                    })
                    .on('error', function(err) {
                        msg.payload = 'Error extracting keyframes: ' + err.message;
                        node.send(msg);
                    })
                    .run();
            });
        });
    }
    RED.nodes.registerType("extract-keyframes", ExtractKeyframesNode);
}


// let videoPath="/Users/michaelding/Downloads/ev.mp4";
//
// exec('which ffmpeg', (err, stdout, stderr) => {
//     if (err || !stdout) {
//        console.log('FFmpeg not found. Please install FFmpeg and ensure it is in your PATH.')
//
//         return;
//     }
//
//     const ffmpegPath = stdout.trim();
//     ffmpeg.setFfmpegPath(ffmpegPath);
//         const outputDir = './out';
//         const frameRate =  1;
//
//         if (!fs.existsSync(outputDir)) {
//             fs.mkdirSync(outputDir, { recursive: true });
//         }
//
//         ffmpeg(videoPath)
//             .output(path.join(outputDir, 'frame-%04d.png'))
//             .outputOptions([
//                 `-vf fps=${frameRate}`
//             ])
//             .on('end', function() {
//                 console.log('Keyframes extracted successfully');
//
//             })
//             .on('error', function(err) {
//                 console.log('Error extracting keyframes: ' + err.message );
//
//             })
//             .run();
// });